@@ -3,14 +3,17 @@ package Moo;
 use Moo::_strictures;
 use Moo::_mro;
 use Moo::_Utils qw(
+  _check_tracked
   _getglob
   _getstash
   _install_coderef
   _install_modifier
+  _install_tracked
   _load_module
   _set_loaded
   _unimport_coderefs
 );
+use Scalar::Util qw(reftype);
 use Carp qw(croak);
 BEGIN {
   our @CARP_NOT = qw(
@@ -22,20 +25,14 @@ BEGIN {
   );
 }
 
-our $VERSION = '2.002_001';
-$VERSION = eval $VERSION;
+our $VERSION = '2.003_005';
+$VERSION =~ tr/_//d;
 
 require Moo::sification;
 Moo::sification->import;
 
 our %MAKERS;
 
-sub _install_tracked {
-  my ($target, $name, $code) = @_;
-  $MAKERS{$target}{exports}{$name} = $code;
-  _install_coderef "${target}::${name}" => "Moo::${name}" => $code;
-}
-
 sub import {
   my $target = caller;
   my $class = shift;
@@ -85,9 +82,15 @@ sub import {
     };
   }
   return if $MAKERS{$target}{is_class}; # already exported into this package
+
   my $stash = _getstash($target);
-  my @not_methods = map { *$_{CODE}||() } grep !ref($_), values %$stash;
-  @{$MAKERS{$target}{not_methods}={}}{@not_methods} = @not_methods;
+  $MAKERS{$target}{non_methods} = {
+    map +($_ => \&{"${target}::${_}"}),
+    grep exists &{"${target}::${_}"},
+    grep !/::\z/ && !/\A\(/,
+    keys %$stash
+  };
+
   $MAKERS{$target}{is_class} = 1;
   {
     no strict 'refs';
@@ -102,7 +105,7 @@ sub import {
 
 sub unimport {
   my $target = caller;
-  _unimport_coderefs($target, $MAKERS{$target});
+  _unimport_coderefs($target);
 }
 
 sub _set_superclasses {
@@ -125,7 +128,6 @@ sub _set_superclasses {
   elsif (!$target->isa('Moo::Object')) {
     Moo->_constructor_maker_for($target);
   }
-  no warnings 'once'; # piss off. -- mst
   $Moo::HandleMoose::MOUSE{$target} = [
     grep defined, map Mouse::Util::find_meta($_), @_
   ] if Mouse::Util->can('find_meta');
@@ -169,7 +171,6 @@ sub _constructor_maker_for {
   return unless $MAKERS{$target};
   $MAKERS{$target}{constructor} ||= do {
     require Method::Generate::Constructor;
-    require Sub::Defer;
 
     my %construct_opts = (
       package => $target,
@@ -194,7 +195,8 @@ sub _constructor_maker_for {
     my $con;
     my @isa = @{mro::get_linear_isa($target)};
     shift @isa;
-    if (my ($parent_new) = grep { *{_getglob($_.'::new')}{CODE} } @isa) {
+    no strict 'refs';
+    if (my ($parent_new) = grep +(defined &{$_.'::new'}), @isa) {
       if ($parent_new eq 'Moo::Object') {
         # no special constructor needed
       }
@@ -230,19 +232,30 @@ sub _constructor_maker_for {
 }
 
 sub _concrete_methods_of {
-  my ($me, $role) = @_;
-  my $makers = $MAKERS{$role};
-  # grab role symbol table
-  my $stash = _getstash($role);
-  # reverse so our keys become the values (captured coderefs) in case
-  # they got copied or re-used since
-  my $not_methods = { reverse %{$makers->{not_methods}||{}} };
-  +{
-    # grab all code entries that aren't in the not_methods list
-    map {
-      my $code = *{$stash->{$_}}{CODE};
-      ( ! $code or exists $not_methods->{$code} ) ? () : ($_ => $code)
-    } grep !ref($stash->{$_}), keys %$stash
+  my ($me, $class) = @_;
+  my $makers = $MAKERS{$class};
+
+  my $non_methods = $makers->{non_methods} || {};
+  my $stash = _getstash($class);
+
+  my $subs = {
+    map {;
+      no strict 'refs';
+      ${"${class}::${_}"} = ${"${class}::${_}"};
+      ($_ => \&{"${class}::${_}"});
+    }
+    grep exists &{"${class}::${_}"},
+    grep !/::\z/,
+    keys %$stash
+  };
+
+  my %tracked = map +($_ => 1), _check_tracked($class, [ keys %$subs ]);
+
+  return {
+    map +($_ => \&{"${class}::${_}"}),
+    grep !($non_methods->{$_} && $non_methods->{$_} == $subs->{$_}),
+    grep !exists $tracked{$_},
+    keys %$subs
   };
 }
 
@@ -311,7 +324,8 @@ optimised for rapid startup.
 
 C<Moo> avoids depending on any XS modules to allow for simple deployments.  The
 name C<Moo> is based on the idea that it provides almost -- but not quite --
-two thirds of L<Moose>.
+two thirds of L<Moose>.  As such, the L<Moose::Manual> can serve as an effective
+guide to C<Moo> aside from the MOP and Types sections.
 
 Unlike L<Mouse> this module does not aim at full compatibility with
 L<Moose>'s surface syntax, preferring instead to provide full interoperability
@@ -379,11 +393,11 @@ global and turns the mechanism off entirely so don't put this in library code.
 
 =head1 MOO AND CLASS::XSACCESSOR
 
-If a new enough version of L<Class::XSAccessor> is available, it
-will be used to generate simple accessors, readers, and writers for
-better performance.  Simple accessors are those without lazy defaults,
-type checks/coercions, or triggers.  Readers and writers generated
-by L<Class::XSAccessor> will behave slightly differently: they will
+If a new enough version of L<Class::XSAccessor> is available, it will be used
+to generate simple accessors, readers, and writers for better performance.
+Simple accessors are those without lazy defaults, type checks/coercions, or
+triggers.  Simple readers are those without lazy defaults. Readers and writers
+generated by L<Class::XSAccessor> will behave slightly differently: they will
 reject attempts to call them with the incorrect number of parameters.
 
 =head1 MOO VERSUS ANY::MOOSE
@@ -401,7 +415,7 @@ dependencies but is also fully usable by L<Moose> users, you should be using
 L<Moo>.
 
 For a full explanation, see the article
-L<http://shadow.cat/blog/matt-s-trout/moo-versus-any-moose> which explains
+L<https://shadow.cat/blog/matt-s-trout/moo-versus-any-moose> which explains
 the differing strategies in more detail and provides a direct example of
 where L<Moo> succeeds and L<Any::Moose> fails.
 
@@ -443,8 +457,13 @@ superclasses.
   my $meta = Foo::Bar->meta;
   my @methods = $meta->get_method_list;
 
-Returns a L<Moose metaclass|Moose::Meta::Class> object for the class.  The
-metaclass will only be built on demand, loading Moose in the process.
+Returns an object that will behave as if it is a
+L<Moose metaclass|Moose::Meta::Class> object for the class. If you call
+anything other than C<make_immutable> on it, the object will be transparently
+upgraded to a genuine L<Moose::Meta::Class> instance, loading Moose in the
+process if required. C<make_immutable> itself is a no-op, since we generate
+metaclasses that are already immutable, and users converting from Moose had
+an unfortunate tendency to accidentally load Moose by calling it.
 
 =head1 LIFECYCLE METHODS
 
@@ -581,7 +600,7 @@ The options for C<has> are as follows:
 
 =over 2
 
-=item * C<is>
+=item C<is>
 
 B<required>, may be C<ro>, C<lazy>, C<rwp> or C<rw>.
 
@@ -605,7 +624,7 @@ This feature comes from L<MooseX::AttributeShortcuts>.
 C<rw> stands for "read-write" and generates a normal getter/setter by
 defaulting the C<accessor> to the name of the attribute specified.
 
-=item * C<isa>
+=item C<isa>
 
 Takes a coderef which is used to validate the attribute.  Unlike L<Moose>, Moo
 does not include a basic type system, so instead of doing C<< isa => 'Num' >>,
@@ -644,7 +663,7 @@ Note that this example is purely illustrative; anything that returns a
 L<Moose::Meta::TypeConstraint> object or something similar enough to it to
 make L<Moose> happy is fine.
 
-=item * C<coerce>
+=item C<coerce>
 
 Takes a coderef which is meant to coerce the attribute.  The basic idea is to
 do something like the following:
@@ -663,7 +682,7 @@ L<Sub::Quote aware|/SUB QUOTE AWARE>
 If the C<isa> option is a blessed object providing a C<coerce> or
 C<coercion> method, then the C<coerce> option may be set to just C<1>.
 
-=item * C<handles>
+=item C<handles>
 
 Takes a string
 
@@ -682,7 +701,7 @@ Takes a hashref
     un => 'one',
   }
 
-=item * C<trigger>
+=item C<trigger>
 
 Takes a coderef which will get called any time the attribute is set. This
 includes the constructor, but not default or built values. The coderef will be
@@ -697,7 +716,7 @@ supported.
 
 L<Sub::Quote aware|/SUB QUOTE AWARE>
 
-=item * C<default>
+=item C<default>
 
 Takes a coderef which will get called with $self as its only argument to
 populate an attribute if no value for that attribute was supplied to the
@@ -714,7 +733,7 @@ existence.
 
 L<Sub::Quote aware|/SUB QUOTE AWARE>
 
-=item * C<predicate>
+=item C<predicate>
 
 Takes a method name which will return true if an attribute has a value.
 
@@ -723,7 +742,7 @@ C<has_${attr_name}> if your attribute's name does not start with an
 underscore, or C<_has_${attr_name_without_the_underscore}> if it does.
 This feature comes from L<MooseX::AttributeShortcuts>.
 
-=item * C<builder>
+=item C<builder>
 
 Takes a method name which will be called to create the attribute - functions
 exactly like default except that instead of calling
@@ -743,7 +762,7 @@ If you set this to a coderef or code-convertible object, that variable will be
 installed under C<$class::_build_${attr_name}> and the builder set to the same
 name.
 
-=item * C<clearer>
+=item C<clearer>
 
 Takes a method name which will clear the attribute.
 
@@ -755,41 +774,41 @@ This feature comes from L<MooseX::AttributeShortcuts>.
 B<NOTE:> If the attribute is C<lazy>, it will be regenerated from C<default> or
 C<builder> the next time it is accessed. If it is not lazy, it will be C<undef>.
 
-=item * C<lazy>
+=item C<lazy>
 
 B<Boolean>.  Set this if you want values for the attribute to be grabbed
 lazily.  This is usually a good idea if you have a L</builder> which requires
 another attribute to be set.
 
-=item * C<required>
+=item C<required>
 
 B<Boolean>.  Set this if the attribute must be passed on object instantiation.
 
-=item * C<reader>
+=item C<reader>
 
 The name of the method that returns the value of the attribute.  If you like
 Java style methods, you might set this to C<get_foo>
 
-=item * C<writer>
+=item C<writer>
 
 The value of this attribute will be the name of the method to set the value of
 the attribute.  If you like Java style methods, you might set this to
 C<set_foo>.
 
-=item * C<weak_ref>
+=item C<weak_ref>
 
 B<Boolean>.  Set this if you want the reference that the attribute contains to
 be weakened. Use this when circular references, which cause memory leaks, are
 possible.
 
-=item * C<init_arg>
+=item C<init_arg>
 
 Takes the name of the key to look for at instantiation time of the object.  A
 common use of this is to make an underscored attribute have a non-underscored
 initialization name. C<undef> means that passing the value in on instantiation
 is ignored.
 
-=item * C<moosify>
+=item C<moosify>
 
 Takes either a coderef or array of coderefs which is meant to transform the
 given attributes specifications if necessary when upgrading to a Moose role or
@@ -885,9 +904,17 @@ Anything imported or declared after will be still be available.
 
   1;
 
-If you were to import C<md5_hex> after L<namespace::clean> you would
-be able to call C<< ->md5_hex() >> on your C<Record> instances (and it
-probably wouldn't do what you expect!).
+For example if you were to import these subroutines after
+L<namespace::clean> like this
+
+  use namespace::clean;
+
+  use Digest::MD5 qw(md5_hex);
+  use Moo;
+
+then any C<Record> C<$r> would have methods such as C<< $r->md5_hex() >>, 
+C<< $r->has() >> and C<< $r->around() >> - almost certainly not what you
+intend!
 
 L<Moo::Role>s behave slightly differently.  Since their methods are
 composed into the consuming class, they can do a little more for you
@@ -903,6 +930,8 @@ using version 0.16 or newer.
 
 =head1 INCOMPATIBILITIES WITH MOOSE
 
+=head2 TYPES
+
 There is no built-in type system.  C<isa> is verified with a coderef; if you
 need complex types, L<Type::Tiny> can provide types, type libraries, and
 will work seamlessly with both L<Moo> and L<Moose>.  L<Type::Tiny> can be
@@ -912,18 +941,12 @@ that you can write
   use Types::Standard qw(Int);
   has days_to_live => (is => 'ro', isa => Int);
 
+=head2 API INCOMPATIBILITIES
+
 C<initializer> is not supported in core since the author considers it to be a
 bad idea and Moose best practices recommend avoiding it. Meanwhile C<trigger> or
 C<coerce> are more likely to be able to fulfill your needs.
 
-There is no meta object.  If you need this level of complexity you need
-L<Moose> - Moo is small because it explicitly does not provide a metaprotocol.
-However, if you load L<Moose>, then
-
-  Class::MOP::class_of($moo_class_or_role)
-
-will return an appropriate metaclass pre-populated by L<Moo>.
-
 No support for C<super>, C<override>, C<inner>, or C<augment> - the author
 considers augment to be a bad idea, and override can be translated:
 
@@ -942,7 +965,7 @@ considers augment to be a bad idea, and override can be translated:
 
 The C<dump> method is not provided by default. The author suggests loading
 L<Devel::Dwarn> into C<main::> (via C<perl -MDevel::Dwarn ...> for example) and
-using C<$obj-E<gt>$::Dwarn()> instead.
+using C<< $obj->$::Dwarn() >> instead.
 
 L</default> only supports coderefs and plain scalars, because passing a hash
 or array reference as a default is almost always incorrect since the value is
@@ -993,6 +1016,18 @@ or, if you're inheriting from a non-Moose class,
   use warnings FATAL => "all";
   use MooseX::AttributeShortcuts;
 
+=head2 META OBJECT
+
+There is no meta object.  If you need this level of complexity you need
+L<Moose> - Moo is small because it explicitly does not provide a metaprotocol.
+However, if you load L<Moose>, then
+
+  Class::MOP::class_of($moo_class_or_role)
+
+will return an appropriate metaclass pre-populated by L<Moo>.
+
+=head2 IMMUTABILITY
+
 Finally, Moose requires you to call
 
   __PACKAGE__->meta->make_immutable;
@@ -1006,15 +1041,10 @@ to Moo by providing a more Moose-like interface.
 
 =head1 SUPPORT
 
-Users' IRC: #moose on irc.perl.org
+IRC: #moose on irc.perl.org
 
 =for :html
-L<(click for instant chatroom login)|http://chat.mibbit.com/#moose@irc.perl.org>
-
-Development and contribution IRC: #web-simple on irc.perl.org
-
-=for :html
-L<(click for instant chatroom login)|http://chat.mibbit.com/#web-simple@irc.perl.org>
+L<(click for instant chatroom login)|https://chat.mibbit.com/#moose@irc.perl.org>
 
 Bugtracker: L<https://rt.cpan.org/Public/Dist/Display.html?Name=Moo>
 
@@ -1070,6 +1100,6 @@ as listed above.
 =head1 LICENSE
 
 This library is free software and may be distributed under the same terms
-as perl itself. See L<http://dev.perl.org/licenses/>.
+as perl itself. See L<https://dev.perl.org/licenses/>.
 
 =cut
