@@ -2,19 +2,19 @@ package Moo::Role;
 
 use Moo::_strictures;
 use Moo::_Utils qw(
+  _check_tracked
   _getglob
   _getstash
   _install_coderef
   _install_modifier
+  _install_tracked
   _load_module
   _name_coderef
   _set_loaded
   _unimport_coderefs
 );
-use Sub::Defer ();
-use Sub::Quote qw(quote_sub sanitize_identifier);
-use Role::Tiny ();
 use Carp qw(croak);
+use Role::Tiny ();
 BEGIN { our @ISA = qw(Role::Tiny) }
 BEGIN {
   our @CARP_NOT = qw(
@@ -25,8 +25,8 @@ BEGIN {
   );
 }
 
-our $VERSION = '2.002_001';
-$VERSION = eval $VERSION;
+our $VERSION = '2.003_005';
+$VERSION =~ tr/_//d;
 
 require Moo::sification;
 Moo::sification->import;
@@ -44,12 +44,6 @@ our %APPLY_DEFAULTS;
 our %COMPOSED;
 our @ON_ROLE_CREATE;
 
-sub _install_tracked {
-  my ($target, $name, $code) = @_;
-  $INFO{$target}{exports}{$name} = $code;
-  _install_coderef "${target}::${name}" => "Moo::Role::${name}" => $code;
-}
-
 sub import {
   my $target = caller;
   if ($Moo::MAKERS{$target} and $Moo::MAKERS{$target}{is_class}) {
@@ -59,6 +53,14 @@ sub import {
   goto &Role::Tiny::import;
 }
 
+sub _accessor_maker_for {
+  my ($class, $target) = @_;
+  ($INFO{$target}{accessor_maker} ||= do {
+    require Method::Generate::Accessor;
+    Method::Generate::Accessor->new
+  });
+}
+
 sub _install_subs {
   my ($me, $target) = @_;
   _install_tracked $target => has => sub {
@@ -71,10 +73,8 @@ sub _install_subs {
     my %spec = @_;
     foreach my $name (@name_proto) {
       my $spec_ref = @name_proto > 1 ? +{%spec} : \%spec;
-      ($INFO{$target}{accessor_maker} ||= do {
-        require Method::Generate::Accessor;
-        Method::Generate::Accessor->new
-      })->generate_method($target, $name, $spec_ref);
+      $me->_accessor_maker_for($target)
+        ->generate_method($target, $name, $spec_ref);
       push @{$INFO{$target}{attributes}||=[]}, $name, $spec_ref;
       $me->_maybe_reset_handlemoose($target);
     }
@@ -113,7 +113,7 @@ sub meta {
 
 sub unimport {
   my $target = caller;
-  _unimport_coderefs($target, $INFO{$target});
+  _unimport_coderefs($target);
 }
 
 sub _maybe_reset_handlemoose {
@@ -123,6 +123,19 @@ sub _maybe_reset_handlemoose {
   }
 }
 
+sub _non_methods {
+  my $self = shift;
+  my ($role) = @_;
+
+  my $non_methods = $self->SUPER::_non_methods(@_);
+
+  my $all_subs = $self->_all_subs($role);
+  $non_methods->{$_} = $all_subs->{$_}
+    for _check_tracked($role, [ keys %$all_subs ]);
+
+  return $non_methods;
+}
+
 sub methods_provided_by {
   my ($self, $role) = @_;
   _load_module($role);
@@ -171,6 +184,7 @@ sub _inhale_if_moose {
         my $spec = { %{ $is_mouse ? $attr : $attr->original_options } };
 
         if ($spec->{isa}) {
+          require Sub::Quote;
 
           my $get_constraint = do {
             my $pkg = $is_mouse
@@ -182,9 +196,9 @@ sub _inhale_if_moose {
 
           my $tc = $get_constraint->($spec->{isa});
           my $check = $tc->_compiled_type_constraint;
-          my $tc_var = '$_check_for_'.sanitize_identifier($tc->name);
+          my $tc_var = '$_check_for_'.Sub::Quote::sanitize_identifier($tc->name);
 
-          $spec->{isa} = quote_sub
+          $spec->{isa} = Sub::Quote::quote_sub(
             qq{
               &${tc_var} or Carp::croak "Type constraint failed for \$_[0]"
             },
@@ -192,7 +206,7 @@ sub _inhale_if_moose {
             {
               package => $role,
             },
-          ;
+          );
 
           if ($spec->{coerce}) {
 
@@ -261,7 +275,9 @@ sub _make_accessors {
 
 sub _undefer_subs {
   my ($self, $target, $role) = @_;
-  Sub::Defer::undefer_package($role);
+  if ($INC{'Sub/Defer.pm'}) {
+    Sub::Defer::undefer_package($role);
+  }
 }
 
 sub role_application_steps {
@@ -305,7 +321,7 @@ sub create_class_with_roles {
       and $m = Moo->_accessor_maker_for($superclass)
       and ref($m) ne 'Method::Generate::Accessor') {
     # old fashioned way time.
-    *{_getglob("${new_name}::ISA")} = [ $superclass ];
+    @{*{_getglob("${new_name}::ISA")}{ARRAY}} = ($superclass);
     $Moo::MAKERS{$new_name} = {is_class => 1};
     $me->apply_roles_to_package($new_name, @roles);
   }
@@ -318,6 +334,7 @@ sub create_class_with_roles {
   if ($INC{'Moo/HandleMoose.pm'} && !$Moo::sification::disabled) {
     Moo::HandleMoose::inject_fake_metaclass_for($new_name);
   }
+  $COMPOSED{class}{$new_name} = 1;
   _set_loaded($new_name, (caller)[1]);
   return $new_name;
 }
@@ -336,13 +353,11 @@ sub apply_roles_to_object {
         and keys %attrs
         and my $con_gen = Moo->_constructor_maker_for($class)
         and my $m = Moo->_accessor_maker_for($class)) {
-      require Sub::Quote;
 
       my $specs = $con_gen->all_attribute_specs;
 
       my %captures;
       my $code = join('',
-        "no warnings 'void';\n",
         ( map {
           my $name = $_;
           my $spec = $specs->{$name};
@@ -361,15 +376,21 @@ sub apply_roles_to_object {
           }
         } sort keys %attrs ),
       );
-      Sub::Quote::quote_sub(
-        "${class}::_apply_defaults",
-        $code,
-        \%captures,
-        {
-          package => $class,
-          no_install => 1,
-        }
-      );
+      if ($code) {
+        require Sub::Quote;
+        Sub::Quote::quote_sub(
+          "${class}::_apply_defaults",
+          "no warnings 'void';\n$code",
+          \%captures,
+          {
+            package => $class,
+            no_install => 1,
+          }
+        );
+      }
+      else {
+        0;
+      }
     }
     else {
       0;
@@ -485,8 +506,9 @@ And elsewhere:
 =head1 DESCRIPTION
 
 C<Moo::Role> builds upon L<Role::Tiny>, so look there for most of the
-documentation on how this works.  The main addition here is extra bits to make
-the roles more "Moosey;" which is to say, it adds L</has>.
+documentation on how this works (in particular, using C<Moo::Role> also
+enables L<strict> and L<warnings>).  The main addition here is extra bits to
+make the roles more "Moosey;" which is to say, it adds L</has>.
 
 =head1 IMPORTED SUBROUTINES
 
